// $Id: cgi.C,v 1.16 2003/10/07 12:24:42 dvermeir Exp $
#include <sstream>

#include <dvutil/filterstreambuf.h>
#include <dvutil/logstream.h>
#include <dvnet/inetaddress.h>

#include <config.h>
#include "cgidata.h"
#include "formdata.h"
#include "cookieprops.h"
#include "wwwdecode.h"
#include "cgi.h"

namespace Dv { namespace Cgi {

//////////////////////////////////////////////////////////////////////////////
// Cgi::Filter ensures that output is preceeded by (HTTP) header_.
//////////////////////////////////////////////////////////////////////////////

Cgi::Filter::Filter(Cgi& cgi): count_(0), cgi_(cgi) {
}

int
Cgi::Filter::get() {
return EOF;
}

int
Cgi::Filter::put(int c) {
if (c!=EOF) {
  if (count_==0) {
    if ( cgi_.header().debug() )
      cgi_.log() << "Dv::Cgi::HttpHeader::Debug:\n" << cgi_.header();
    std::cout << cgi_.header();
    }
  ++count_;
  return (std::cout.put(c)?c:EOF);
  }
return EOF;
}

int
Cgi::Filter::sync() {
return (std::cout.rdbuf()->STREAMBUF_PUBSYNC() == 0? 0 : -1);
}

//////////////////////////////////////////////////////////////////////////////
// Cgi
//////////////////////////////////////////////////////////////////////////////

Cgi::Cgi(const std::string& name, const std::string& format, bool debug) throw (): std::ostream(0), 
  name_(name), filter_(*this), data_(), log_(std::cerr,name), here_(),
  here_base_(), client_host_(), this_host_(), header_(format, debug) {
try {
  rdbuf(new Dv::Util::filterstreambuf<Filter>(filter_));
  cgidata(data_); // never 0, cgidata() may throw exception
  if (!formdata(data_,formdata_))
    throw CgiError("formdata failure");
  cookieprops(cookies_);
  Dv::Util::env2props(env_);

  here_ = env("REQUEST_URI"); // includes possible parameters
  
  std::string::size_type qmark(here_.find('?'));
  if ( qmark == std::string::npos )
    here_base_ = here_;
  else
    here_base_ = here_.substr(0, qmark);

  client_host_ = env("REMOTE_ADDR"); // dot address, hopefully
  if (client_host_ == "127.0.0.1") 
    client_host_ = 
      Dv::Net::InetAddress::by_name(Dv::Net::InetAddress::local_host())->dot_address();
  client_host_name_ = Dv::Net::InetAddress::by_name(client_host_)->host();

  this_host_ = env("SERVER_ADDR"); // dot address
  if ( env()("SERVER_NAME") )
    this_host_name_ = env("SERVER_NAME");
  else
    this_host_name_ = Dv::Net::InetAddress::by_name(this_host_)->host();
  }
catch (std::exception& e) {
  std::cerr << "Cgi::Cgi::Cgi(" << format << ") error: " << e.what() << std::endl;
  }
}

Cgi::~Cgi() {
if (filter_.count()==0) {
  const std::string title("Dv::Cgi::Cgi::~Cgi error");
  header_.content_type("text/html");
  (*this) << "<html><head><title>\n" 
          << title  
	  << "</title></head>\n<body bgcolor=\"#ffffff\">\n"
          << "<h2>" << title << "</h2>\n" 
          << "<p> The destructor for a <i>Cgi</i> object was called without\n"
	     "having written any output to it.\n\n<p>The error url was"
          << "<code>" << env("REQUEST_URI") << "</code>"
          << "<p> This is an internal error in the application's use\n"
             "of the <i>Dv::Cgi::Cgi</i> class in the\n\n"
             "<a href=\"http://tinf2.vub.ac.be/~dvermeir/software/dv/index.html\">\n"
             "dvcgi</a> package.\n"
          << "<hr>\n"
             "<a href=\"mailto:dvermeir@vub.ac.be\">Dirk Vermeir</a>\n"
	     "</body></html>"
          << std::endl;
  }
rdbuf()->STREAMBUF_PUBSYNC();
delete rdbuf(); 
}

std::string
Dv::Cgi::Cgi::props_as_url_args(const Dv::Util::Props& props) {
std::ostringstream oss;
for (Dv::Util::Props::iterator i=props.begin(); i!=props.end(); ++i) {
  if (i != props.begin() )
    oss << '&';
  std::string key;
  Dv::Cgi::www_encode((*i).first, key);
  std::string value;
  Dv::Cgi::www_encode(*((*i).second), value);
  oss << key << '=' << value;
  }
return oss.str();
}

std::string
Cgi::url(const std::string& url, const Dv::Util::Props& props) {
if (! props.size() )
  return url;
return url +
  ( (url.find('?')==std::string::npos) ? "?" : "&" ) + props_as_url_args(props);
}

std::string
Dv::Cgi::Cgi::url(const Dv::Util::Props& props) const {
return url(here_base(), props);
}

void
Dv::Cgi::Cgi::debug(std::ostream& os) const {
os << "env:\n" << env() << "\nprops:\n" << props()
  << "\ncookies:\n" << cookies() << std::endl;
}

}}
