// $Id: example-session.C,v 1.1 2004/03/20 14:20:44 dvermeir Exp $
// Silly example program using Dv::Cgi::SessionServer

#include <dvcgi/sessionserver.h>

const char* html_prefix = 
  "<html>\n"
  "<head><title>test-sessionserver</title></head>\n"
  "<body bgcolor=\"#ffffff\">\n"
  ;

const char* html_suffix = 
  "</body>\n"
  "</html>\n"
  ;

const char* sum_form = 
  "<form>\n"
  "<input name=\"number\" type=\"text\" value=\"2\">\n"
  "<input name=\"operation\" type=\"submit\" value=\"add\">\n"
  "<input name=\"operation\" type=\"submit\" value=\"stop\">\n"
  "</form>\n"
  ;

const char* login_form = 
  "<form>\n"
  "name: <input name=\"name\" type=\"text\" value=\"\">\n"
  "password: <input name=\"password\" type=\"password\" value=\"\">\n"
  "<input name=\"operation\" type=\"submit\" value=\"login\">\n"
  "</form>\n"
  ;

const char* main_menu = "<ul>\n"
  "<li><a href=\"?choice=1\">choice 1</a>\n"
  "<li><a href=\"?choice=2\">choice 2</a>\n"
  "<li><a href=\"?choice=3\">quit</a>\n"
  "</ul>\n";


std::string
login(Dv::Cgi::SessionServer& client) {
while (client) { // repeat until valid username/password found
  client << html_prefix << "<h2>login</h2>\n" 
    << login_form << html_suffix << Dv::Cgi::eof;

  if (client("name")) {
    std::string uname = client.props("name");
    // check password
    client.log() << "login: " << uname << std::endl;
    if ( uname.size() )
      return uname;
    }
  }
throw std::runtime_error("login error");
}

int
update_sum(Dv::Cgi::SessionServer& client, int initial_sum) {
int sum = initial_sum;
while (client) {
  client << html_prefix << "<h2>choice 1</h2>\n" 
         << "<p>sum = " << sum << "<p>" << sum_form << html_suffix << Dv::Cgi::eof;

  if (client("operation")) {
    std::string operation;
    int n;

    client.props() >> "operation" >> operation >> "number" >> n;
    if (operation=="add") 
      sum += n;
    else if (operation=="stop") 
      return sum; // exit 
    else
      throw std::runtime_error(std::string("illegal operation:'")+operation+"'");
    }
  }
return 0;
}

static const std::string HERE = "./example-session.cgi";
static const std::string TIMEOUT_URL = HERE;
static const std::string ERROR_URL = "http://tinf2.vub.ac.be/index.html";

int 
main(int,char**) {
Dv::Cgi::SessionServer client("example-session", 10*1000, TIMEOUT_URL, ERROR_URL);
int sum1 = 0;
int sum2 = 0;
client.log() << "main()" << std::endl;
try {
  // Header is "global" for all interactions.
  client.header().content_type("text/html").nocache();

  // First log in.
  std::string user_name = login(client);

  while (client) {
    // Select a meny item, exit if "quit" (choice 3) is selected.
    client << html_prefix << "<h2>" << user_name << "</h2>\n"
           << main_menu << html_suffix << Dv::Cgi::eof;

    if (client("choice")) {
      int choice;
      client.props() >> "choice" >> choice;
      switch (choice) {
        case 1: 
	  sum1 = update_sum(client, sum1);
	  break;
        case 2:
	  sum2 = update_sum(client, sum2);
	  break;
        case 3:
	  client.disconnect(HERE);
	  break;
        default: 
          throw std::runtime_error("bad choice");
        }
      }
    else
      throw std::runtime_error("no choice");
    }
  client.disconnect(HERE);
  }
catch (std::runtime_error& e) {
  if (client) {
    client.header().content_type("text/plain").nocache();
    client << "exception = " << e.what() << std::endl;
    }
  client.log() << "exception = " << e.what() << std::endl;
  client.disconnect();
  }
}
