// $Id: formdata.C,v 1.9 2005/01/10 11:05:59 dvermeir Exp $

#include	<iostream>
#include	<dvutil/props.h>
#include	<dvutil/date.h>

#include	"formdata.h"
#include	"wwwdecode.h"


namespace Dv { namespace Cgi {

using namespace std;
using Dv::Util::Props;

/**
 * Like getline(istream&, string& s). The '\n', and a preceding '\r'
 * is ``read'' but not stored in s.
 *
 * \param s string to read from: reading will start at \a s[from]
 * \param from position in \a s to start reading from, if
 *   from == string::npos, line is cleared and string::npos is
 *   returned
 * \param line string for result
 * \return position right after newline delimiting read line or
 *   string::npos if no '\n' was found.
 */
static string::size_type
getline_nocr(const string& s, string::size_type from, string& line) {
if (from == string::npos) {
  line = "";
  return from;
  }
string::size_type eol = s.find('\n',from);
if (eol != string::npos)
  line = string(s, from, eol - from);
else
  line = string(s,from);

int l(line.size());
if (l)
  if (line[l-1]=='\r')
    line.erase(l-1);
return (eol==string::npos ? eol : eol+1);
}

static string // extract string in data between separator1 and separator2
extract(const string& data, const string& separator1, const string& separator2) {
string result;
string::size_type begin(data.find(separator1, 0));
if (begin != string::npos) {
  begin += separator1.size();
  if (begin<data.size()) {
    string::size_type end = data.find(separator2,begin);
    if (end != string::npos)
      result = data.substr(begin, end - begin);
    }
  }
return result;
}

// AASDASD=ASDASDAS&ASDASDA=ASDASDA&
// 01234567890123456789012345678901234567890123456789
//           1         2         3         4
//
// begin = 17
// end = 32
// equals = 24

static void // analyze key=value string and add(key,value) to props
keyvalue(const string& data, string::size_type begin,
  string::size_type end, Props& props) throw() {
// pre: begin = start of ``key=value'', a valid position in data, i.e.
//		begin >=0, begin < data.size()
//      end   = 1 position past ``key=value'', 
//		(end-1) must be a valid position in data!
// thus data[begin..end[ contains ``key=value''
//
string::size_type equals(data.find('=', begin)); // ok because begin is valid
if ( (equals != string::npos) && (equals < end) ) {
  string key(data, begin, equals - begin);
  string value(data, equals+1, end - equals -1);
  props.append(www_decode(key), www_decode(value), ",");
  }
}

bool
urlencoded_formdata(const string& data,Props& props) throw () {
string::size_type begin = 0;
string::size_type ampersand = string::npos;
while ((begin < data.size()) && 
       (ampersand = data.find('&', begin)) != string::npos) {
  // begin valid && ampersand valid from while condition
  keyvalue(data, begin, ampersand, props);
  begin = ampersand + 1;
  }
if ( (begin < data.size()) && ( data.size() > 0) )
  // begin valid && data.size()-1 valid from if condition
  keyvalue(data, begin, data.size(), props); // last key-value pair
return true;
}



bool
multipart_formdata(const string& data,Props& props) throw () {
static const string DISPOSITION("Content-Disposition: ");
static const string NAME("name=\"");
static const string FILENAME("filename=\"");
static const string SEMICOLON(";");
static const string DQUOTE("\"");

// Layout of key,value in multipart form data:
// 
// 1. First line is boundary
// 2. The second main header line contains ``disposition'', 
//    ``name'' and optionally ``filename''
// 3. A number of optional header lines like `Content-Type: ...' 
// 4. An empty line
// 5. The data associated with the key, i.e. the value
// 6. A line starting with boundary
// 7. [next key-value block]
string::size_type start(0);

// 1. Read boundary.
string	boundary;
start = getline_nocr(data, start, boundary);

if (boundary.size() == 0)
  return false;

// Since we need to search for boundary at the beginning of a line and
// never at the beginning of the file, we'll keep a version of
// boundary with a prepended '\n'.
string nl_boundary("\n");
nl_boundary += boundary;

string	header;
while ((start=getline_nocr(data,start,header))!=string::npos) {
  // 2. Read main header line.
  string disposition(extract(header,DISPOSITION,SEMICOLON));
  string key(extract(header,NAME,DQUOTE));
  string filename(extract(header,FILENAME,DQUOTE));
  
  cerr << "multipart_formdata: key=" << key << endl;
  // 3,4. Skip further header lines until empty line.
  while (header.size())
    start = getline_nocr(data, start, header);

  // 5. Cursor is right after empty line, i.e. at the
  //    start of the data. Search for nl_boundary.
  string::size_type eodata = data.find(nl_boundary, start);
  if (eodata == string::npos)
    return false; // no boundary found

  // Eodata points to the \n at the start of nl_boundary. However,
  // there may be a '\r' before that, which is not part of the
  // data [THIS MAY NOT BE THE CASE IF CGI WAS ACTIVATED USING
  // THE GET METHOD]. We use pos to point to this '\r'.

  string::size_type pos(eodata);
  if (pos>0)
    if (data[pos-1]=='\r')
      --pos;
  string value(data, start, pos - start);

  // Get rest of line that stared with boundary.
  start = eodata + nl_boundary.size();
  if (start >= data.size())
    start = string::npos;
  // Read junk until start of next line.
  string junk;
  start = getline_nocr(data,start,junk);

  cerr << "multipart_formdata: value.size()=" << value.size() << endl;
  if (value.size() < 100)
    cerr << "multipart_formdata: value=" << value << endl;

  props.add(www_decode(key), value);
  if (filename.size())
    props.add(key+"-filename",www_decode(filename));
  }
return true;
}

bool
formdata(const string& data, Props& props) throw () {
bool multipart(true);
if (data.size() < 2) 
  return urlencoded_formdata(data,props);
for (unsigned int i=0;(i<2);++i)
  if (data[i]!='-')
    multipart = false;
if (multipart)
  return multipart_formdata(data,props);
else
  return urlencoded_formdata(data,props);
}

}}
