#ifndef	DV_FORMDATA_H
#define	DV_FORMDATA_H

// $Id: formdata.h,v 1.10 2004/04/27 12:58:08 dvermeir Exp $

#include	<iostream>
#include	<dvutil/props.h>

/** @file
 *  This file defines functions to retrieve name-value form data pairs
 *  from a C string (which is probably obtained using Dv::Cgi::cgidata):
 *  <ul>
 *  <li> Dv::Cgi::urlencoded_formdata(const char* data,Dv::Util::Props&)
 *  <li> Dv::Cgi::multipart_formdata(const char* data,Dv::Util::Props&)
 *  <li> Dv::Cgi::formdata(const char* data,Dv::Util::Props&)
 *  </ul>
*/
namespace Dv { namespace Cgi {

/**
 * Convert url encoded form data to <key,value> pairs in Dv::Util::Props.
 *
 * @param data raw data, typically the result of calling Cgi::cgidata().
 * @param props where key=value pairs will be stored. 
 * @return true unless something went wrong.
 * @warning several values for the same keys will be appended to the
 *   value associated with the key, separated by comma's.
 * @see Dv::Util::Props
 */
bool urlencoded_formdata(const std::string& data, Dv::Util::Props& props) throw ();
/**
 * Convert multipart encoded form data to <key,value> pairs in Dv::Util::Props.
 *
 * @param data raw data, typically the result of calling Cgi::cgidata().
 * @param props where key=value pairs will be stored. For a ``file''
 *   field with name ``myfile'' in a form, \a props[myfile] will
 *   contain the contents of the file while \a props[myfile-name] will
 *   contain the filename, if any was transmitted.
 * @return true unless something went wrong.
 * @warning several values for the same keys will be appended to the
 *   value associated with the key, separated by comma's.
 * @see Dv::Util::Props
 */
bool multipart_formdata(const std::string& data, Dv::Util::Props& props) throw ();
/**
 * Convert formdata using either urlencoded_formdata or
 * multipart_formdata.
 * @param data raw data, typically the result of calling Cgi::cgidata().
 * @param props where key=value pairs will be stored. 
 * @return true unless something went wrong.
 * @warning several values for the same keys will be appended to the
 *   value associated with the key, separated by comma's.
 * @see Dv::Util::Props
 */
bool formdata(const std::string& data, Dv::Util::Props& props) throw ();

}}
#endif
