// $Id: httphead.C,v 1.9 2003/08/16 08:30:34 dvermeir Exp $
#include <string>
#include <list>
#include <iterator>
#include <sstream>

#include "httphead.h"


namespace Dv {
namespace Cgi {


HttpHeader::HttpHeader(const std::string& ctype, bool debug): debug_(debug) {
content_type(ctype);
}

static const std::string CONTENT_TYPE("Content-type: ");

static void 
print_cookie(std::ostream& os,const std::string& name, const std::string& value,
  const std::string& path) {
os << "Set-Cookie: ";
if (value.size())
  os << name << "=" << value << "; path=" << path << ";\n";
else // delete cookie
  os << name << "=deleted; path=" << path 
     << "; expires=Wed, 26-Jul-1997 05:00:00 GMT\n";
}

HttpHeader&
HttpHeader::cookie(const std::string& name,const std::string&
value,const std::string& path) {
cookies_[name]=make_pair(value,path);
return *this;
}

HttpHeader&
HttpHeader::add(const std::string& name,const std::string& value) {
headers_[name] = value;
return *this;
}

HttpHeader&
HttpHeader::erase(const std::string& name) {
headers_.erase(name);
return *this;
}

HttpHeader&
HttpHeader::location(const std::string& value) {
return add("Location: ", value);
}

HttpHeader&
HttpHeader::refresh(size_t duration, const std::string& value) {
std::ostringstream oss;
oss << duration;
if ( value.size() )
  oss << "; URL=" << value;
return add("Refresh: ", oss.str());
}

HttpHeader&
HttpHeader::nocache() {
// should also send last-modified now
//       "Last-Modified: " . gmdate("D, d M Y H:i:s\n") . "GMT"); // always modified
// return "Expires: Mon, 26 Jul 1997 05:00:00 GMT\n"
add("Expires: ","Wed, 26 Jul 1997 05:00:00 GMT");
add("Cache-Control: ","no-cache, must-revalidate"); // HTTP/1.1
add("Pragma: ","no-cache"); // HTTP/1.0
return *this;
}

HttpHeader&
HttpHeader::content_type(const std::string& value) {
return add("Content-type: ",value);
}

std::ostream&
operator<<(std::ostream& os,const HttpHeader& h) {
for (HttpHeader::cookie_map::const_iterator i=h.cookies_.begin();(i!=h.cookies_.end());++i) 
  print_cookie(os,(*i).first,(*i).second.first,(*i).second.second);
for (HttpHeader::const_iterator i=h.begin();(i!=h.end());++i)
  os << (*i).first << (*i).second << "\n";
return os << std::endl;
}

}}
